#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_manipulating_config_between_reload(config, syslog_ng):
    file_source = config.create_file_source(file_name="input.log")
    file_destination = config.create_file_destination(file_name="output.log")
    destination_group = config.create_statement_group(file_destination)

    logpath = config.create_logpath(statements=[file_source, destination_group])

    syslog_ng.start(config)

    # update positional value of file source
    file_source.set_path("updated_input.log")

    # add new option to file source
    file_source.options["log_iw_size"] = "100"

    # create new file source and add to separate source group
    file_source2 = config.create_file_source(file_name="input2.log")
    source_group2 = config.create_statement_group(file_source2)

    # create new file destination and update first destination group
    file_destination2 = config.create_file_destination(file_name="output2.log")
    destination_group.append(file_destination2)

    # update first logpath group with new source group
    logpath.add_group(source_group2)

    syslog_ng.reload(config)

    # remove option from file source
    file_source.options.pop("log_iw_size")

    # remove file destination from destination group
    destination_group.remove(file_destination2)

    # remove second source group from logpath
    logpath.logpath.remove(source_group2)

    syslog_ng.reload(config)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
INPUT_MESSAGES = "PROXY TCP4 1.1.1.1 2.2.2.2 3333 4444\r\n" \
                 "message 0\n" \
                 "message 1\n" \
                 "message 2\n"
EXPECTED_MESSAGE0 = "1.1.1.1 2.2.2.2 3333 4444 4 message 0\n"
EXPECTED_MESSAGE1 = "1.1.1.1 2.2.2.2 3333 4444 4 message 1\n"
EXPECTED_MESSAGE2 = "1.1.1.1 2.2.2.2 3333 4444 4 message 2\n"


def test_pp_reload(config, syslog_ng, loggen, port_allocator):
    network_source = config.create_network_source(ip="localhost", port=port_allocator(), transport='"proxied-tcp"', flags="no-parse")
    file_destination = config.create_file_destination(file_name="output.log", template=TEMPLATE)
    config.create_logpath(statements=[network_source, file_destination])

    syslog_ng.start(config)

    network_source.write_log(INPUT_MESSAGES, rate=1)

    # With the current loggen implementation there is no way to properly timing messages.
    # Here I made an assumption that with rate=1, there will be messages which will arrive
    # into syslog-ng AFTER the reload. Timing the reload after the first message arrives.

    # Note: The worst case scenario in case of extreme slowness in the test environment, is
    # that syslog-ng will receive all the messages before reload. In this case the test will
    # not fill it's purpose, and do not test if the headers are reserved between reloads.
    # But at least the test wont be flaky, it will pass in this corner case too.

    assert file_destination.read_log() == EXPECTED_MESSAGE0

    syslog_ng.reload(config)

    assert file_destination.read_log() == EXPECTED_MESSAGE1
    assert file_destination.read_log() == EXPECTED_MESSAGE2



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.helpers.secure_logging.conftest import *  # noqa:F403, F401

seqnum = "$(iterate $(+ 1 $_) 0)"
message_base = "example-message"
example_message = "{}: {}".format(message_base, seqnum)
num_of_messages = 3


def test_secure_logging(config, syslog_ng, slog):
    output_file_name = "output.log"
    generator_source = config.create_example_msg_generator_source(num=num_of_messages, template=config.stringify(example_message), freq="0")
    secure_log_template = "$(slog -k {} -m {} $MSG)".format(slog.derived_key, slog.cmac)
    file_destination = config.create_file_destination(file_name=output_file_name, template=config.stringify(secure_log_template + '\n'))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)

    logs = file_destination.read_logs(num_of_messages)
    # test for no clear text
    assert not any(map(lambda x: message_base in x, logs))

    decrypted = slog.decrypt(output_file_name)
    assert decrypted == ["{}: {}: {}".format(str(i).zfill(20), message_base, i) for i in range(num_of_messages)]



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.common.blocking import wait_until_true
from src.syslog_ng_config.renderer import render_statement


def create_config(config, test_config, msg="foobar"):
    file_true = config.create_file_destination(file_name="dest-true.log", template="'$MSG\n'")
    file_false = config.create_file_destination(file_name="dest-false.log", template="'$MSG\n'")
    file_after = config.create_file_destination(file_name="dest-after.log", template="'$MSG\n'")
    file_fallback = config.create_file_destination(file_name="dest-fallback.log", template="'$MSG\n'")

    preamble = f"""
@version: {config.get_version()}

options {{ stats(level(1)); }};

block filter true() {{
    "1" eq "1"
}};

block filter false() {{
    "0" eq "1"
}};

source genmsg {{
    example-msg-generator(num(1) template("{msg}"));
}};

destination dest_after {{
    {render_statement(file_after)};
}};

destination dest_true {{
    {render_statement(file_true)};
}};

destination dest_false {{
    {render_statement(file_false)};
}};

destination dest_fallback {{
    {render_statement(file_fallback)};
}};

log {{
    source(genmsg);
    destination(dest_fallback);
    flags(fallback);
}};

"""
    config.set_raw_config(preamble + test_config)
    return (file_true, file_false, file_after, file_fallback)


def test_simple_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if (true()) {
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_simple_if_negated(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if (false()) {
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_true.get_stats()
    assert file_false.get_stats()["processed"] == 1
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()
    assert file_false.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_simple_if_that_drops_in_all_branches(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if (true()) {
        filter { false(); };
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_true.get_stats()
    assert "processed" not in file_false.get_stats()
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_fallback.read_log() == "foobar\n"


def test_compound_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        filter { true(); };
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_compound_if_negated_continues_in_the_false_expr(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        filter { false(); };
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_true.get_stats()
    assert file_false.get_stats()["processed"] == 1
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()
    assert file_false.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_compound_if_that_drops_in_all_branches(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        filter { false(); };
        destination(dest_true);
    } else {
        filter { false(); };
        destination(dest_false);
    };

    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_true.get_stats()
    assert "processed" not in file_false.get_stats()
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_fallback.read_log() == "foobar\n"


def test_compound_if_with_messages_dropped_after_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        filter { true(); };
        destination(dest_true);
    } else {
        destination(dest_false);
    };

    filter { false(); };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_fallback.read_log() == "foobar\n"


def test_three_levels_of_conditionals_drop_after_the_innermost_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        if {
            if {
                destination(dest_true);
            } else {
                destination(dest_false);
            };
            filter { false(); };
            destination(dest_false);
        } else {
            destination(dest_true);
        };
    } else {
        destination(dest_false);
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 2
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_three_levels_of_conditionals_drop_after_the_middle_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        if {
            if {
                destination(dest_true);
            } else {
                destination(dest_false);
            };
        } else {
            destination(dest_false);
        };
        filter { false(); };
        destination(dest_false);
    } else {
        destination(dest_true);
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 2
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_three_levels_of_simple_conditionals_drop_after_the_middle_if(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    if {
        if (true()) {
            if (true()) {
                if (true()) {
                    destination(dest_true);
                } else {
                    destination(dest_false);
                };
            } else {
                destination(dest_false);
            };
            filter { false(); };
            destination(dest_false);
        } else {
            destination(dest_false);
        };
    } else {
        destination(dest_true);
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 2
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_three_levels_of_conditionals_drop_after_the_innermost_junction(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    junction {
        channel {
            junction {
                channel {
                    junction {
                        channel {
                            destination(dest_true);
                            flags(final);
                        };
                        channel {
                            destination(dest_false);
                            flags(final);
                        };
                    };
                    filter { false(); };
                    destination(dest_false);
                    flags(final);
                };
                channel {
                    destination(dest_after);
                    flags(final);
                };
            };
            flags(final);
        };
        channel {
            destination(dest_false);
            flags(final);
        };
    };
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_three_levels_of_conditionals_drop_after_the_middle_junction(config, syslog_ng):
    test_config = """

log {
    source(genmsg);
    junction {
        channel {
            junction {
                channel {
                    junction {
                        channel {
                            destination(dest_true);
                            flags(final);
                        };
                        channel {
                            destination(dest_false);
                            flags(final);
                        };
                    };
                    flags(final);
                };
                channel {
                    destination(dest_false);
                    flags(final);
                };
            };
            filter { false(); };
            destination(dest_false);
            flags(final);
        };
        channel {
            destination(dest_true);
            flags(final);
        };
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 2
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"


def test_consuming_messages_into_correlation_state_does_not_cause_else_branches_to_be_processed(config, syslog_ng):
    test_config = """

log {
    # generate two messages
    source(genmsg);
    source(genmsg);

    rewrite {
        set("PROG" value("PROGRAM"));
    };

    if {
        parser {
            grouping-by(key("$PROGRAM")
                timeout(1)
                trigger("$(context-length)" == 2)
                aggregate(value("MESSAGE", "$MSG correlated\n"))
                inject-mode(aggregate-only));
        };
        destination(dest_true);
    } else {
        destination(dest_false);
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback) = create_config(config, test_config)
    syslog_ng.start(config)

    assert wait_until_true(lambda: "processed" in file_true.get_stats() and file_true.get_stats()["processed"] == 1)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar correlated\n"
    assert file_after.read_log() == "foobar correlated\n"



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_group_lines_parser(config, syslog_ng):

    traceback = """Traceback (most recent call last):
File "./lib/merge-grammar.py", line 62, in <module>
  for line in fileinput.input(openhook=fileinput.hook_encoded("utf-8")):
File "/usr/lib/python3.8/fileinput.py", line 248, in __next__
  line = self._readline()
File "/usr/lib/python3.8/fileinput.py", line 368, in _readline
  return self._readline()
ValueError: This is the exception text at the end
"""

    config.update_global_options(stats_level=1)
    file_source = config.create_file_source(file_name="input.log")
    group_lines_parser = config.create_group_lines_parser(key=config.stringify("foo"), multi_line_mode="smart", timeout=10)

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$(format-welf MESSAGE)\n"))
    config.create_logpath(statements=[file_source, group_lines_parser, file_destination])

    syslog_ng.start(config)
    file_source.write_log('\n'.join(map(lambda line: 'python: ' + line, traceback.split('\n'))))
    file_source.write_log("whatvever: the exception text at the end\n")
    assert file_destination.read_log().strip() == 'MESSAGE="' + traceback.strip().replace('"', "\\\"").replace('\n', '\\n') + '"'



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2021 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

from src.common.blocking import wait_until_true
from src.message_builder.log_message import LogMessage


def generate_messages_with_different_program_fields(bsd_formatter, number_of_all_messages, different_program_fields):
    input_messages = ""
    program_idx = 1
    for message_idx in range(1, number_of_all_messages + 1):
        if program_idx == different_program_fields + 1:
            program_idx = 1
        log_message = LogMessage().program(program_idx).message("message idx: %s" % message_idx)
        input_messages += bsd_formatter.format_message(log_message, add_new_line=True)
        program_idx += 1
    return input_messages


@pytest.mark.parametrize(
    "message_counter, message_rate_by_sec, different_program_fields, rate_limit_rate_by_sec, expected_number_of_matched_messages, expected_number_of_not_matched_messages", [
        # All incoming messages=100, which arrives in one sec where every PROGRAM field is the same. From same PROGRAM fields we accept 100 in one sec. At the end we will have 100 matched and 0 not matched messages.
        (100, 100, 1, 100, 100, 0),
        # All incoming messages=100, which arrives in one sec where every PROGRAM field is the same. From same PROGRAM fields we accept only 1 in one sec. At the end we will have 1 matched and 99 not matched messages.
        (100, 100, 1, 1, 1, 99),
        # All incoming messages=100, which arrives in one sec where we have 5 different PROGRAM fields. From same PROGRAM fields we accept only 1 in one sec. At the end we will have 5 matched and 95 not matched messages.
        (100, 100, 5, 1, 5, 95),
        # All incoming messages=100, which arrives in one sec where we have 5 different PROGRAM fields. From same PROGRAM fields we accept 5 in one sec. At the end we will have 25 matched and 75 not matched messages.
        (100, 100, 5, 5, 25, 75),
    ], ids=["rate_limit_filter_1", "rate_limit_filter_2", "rate_limit_filter_3", "rate_limit_filter_4"],
)
def test_rate_limit_filter_acceptance(config, syslog_ng, port_allocator, bsd_formatter, message_counter, message_rate_by_sec, different_program_fields, rate_limit_rate_by_sec, expected_number_of_matched_messages, expected_number_of_not_matched_messages):
    config.update_global_options(stats_level=3)
    s_network = config.create_network_source(ip="localhost", port=port_allocator())
    f_rate_limit = config.create_rate_limit_filter(template="'${PROGRAM}'", rate=rate_limit_rate_by_sec)
    d_file = config.create_file_destination(file_name="output.log")
    config.create_logpath(statements=[s_network, f_rate_limit, d_file])

    syslog_ng.start(config)

    input_messages = generate_messages_with_different_program_fields(bsd_formatter=bsd_formatter, number_of_all_messages=message_counter, different_program_fields=different_program_fields)
    s_network.write_log(input_messages, rate=message_rate_by_sec)

    assert wait_until_true(lambda: f_rate_limit.get_stats() == {'matched': expected_number_of_matched_messages, 'not_matched': expected_number_of_not_matched_messages})



#!/usr/bin/env python3
#############################################################################
# Copyright (c) 2022 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from setuptools import setup
import platform
import os

install_addons=int(os.getenv('PYMODULES_BUILTINS_ONLY', '0')) == 0

packages_builtin=[
  "syslogng",
  "syslogng.debuggercli",
]
requires_builtin=[]

packages_addons=[
  "syslogng.modules.example",
  "syslogng.modules.kubernetes",
  "syslogng.modules.hypr",
  "syslogng.modules.s3",
]

requires_addons=[
  # kubernetes
  "kubernetes",
  # hypr
  "requests",
  # s3
  "boto3",
]

packages = packages_builtin
requires = []
if install_addons:
  packages = packages + packages_addons
  requires = requires + requires_addons

setup(name='syslogng',
      version='1.0',
      description='syslog-ng Python Core & Modules',
      author='Balazs Scheidler',
      author_email='bazsi77@gmail.com',
      url='https://www.syslog-ng.org',
      package_data={"": ["scl/*"]},
      exclude_package_data={"": ["*~"]},
      packages=packages,
      install_requires=requires)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2019 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

test_parameters_raw = [
    (r"""<189>29: foo: *Apr 29 13:58:40.411: %SYS-5-CONFIG_I: Configured from console by console""", "${.app.name}", "cisco"),
    (r"""<189>29: foo: .Nov 19 13:58:40.411: %SYS-5-CONFIG_I: Configured from console by console""", "${.app.name}", "cisco"),
    (r"""<189>29: foo: Nov 19 13:58:40.411: %SYS-5-CONFIG_I: Configured from console by console""", "${.app.name}", "cisco"),
    (r"""<190>30: foo: *Apr 29 13:58:46.411: %SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 stopped - CLI initiated""", "${.app.name}", "cisco"),
    (r"""<190>31: foo: *Apr 29 13:58:46.411: %SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 started - CLI initiated<189>32: 0.0.0.0: *Apr 29 13:59:12.491: %SYS-5-CONFIG_I: Configured from console by console<189>33: 0.0.0.0: *Apr 29 13:59:26.415: %SYS-5-CONFIG_I: Configured from console by console<189>34: 0.0.0.0: *Apr 29 13:59:56.603: %SYS-5-CONFIG_I: Configured from console by console^[[<189>35: *Apr 29 14:00:16.059: %SYS-5-CONFIG_I: Configured from console by console""", "${.app.name}", "cisco"),
    (r"""<190>32: foo: *Apr 29 13:58:46.411: %SYSMGR-STANDBY-3-SHUTDOWN_START: The System Manager has started the shutdown procedure.""", "${.app.name}", "cisco"),
    (r"""<166>2022-02-16T15:31:53Z na-zy-int-fp1140-p02 : %FTD-6-305012: Teardown dynamic TCP translation from FOO-WAN_IN:10.44.60.80/59877 to FOO-OUTSIDE:6.7.8.9/59877 duration 0:01:01""", "${.app.name}", "cisco"),
    (r"""<134>{"count": 1, "supporting_data": {"data_values": ["x.x.x.x", "user@domain.com"], "data_type": "user"}, "organization_unit": "domain/domain/Domain Users/Enterprise Users", "severity_level": 2, "category": null, "timestamp": 1547421943, "_insertion_epoch_timestamp": 1547421943, "ccl": "unknown", "user": "user@domain.com", "audit_log_event": "Login Successful", "ur_normalized": "user@domain.com", "_id": "936289", "type": "admin_audit_logs", "appcategory": null}""", "${.app.name}", "netskope"),
    (r"""<159>Dec 19 10:48:57 EST 10.203.28.21 vendor=Websense product=Security product_version=7.7.0 action=permitted severity=1 category=153 user=- src_host=10.64.134.74 src_port=62189 dst_host=mail.google.com dst_ip=74.125.224.53 dst_port=443 bytes_out=197 bytes_in=76 http_response=200 http_method=CONNECT http_content_type=- http_user_agent=Mozilla/5.0_(Windows;_U;_Windows_NT_6.1;_enUS;_rv:1.9.2.23)_Gecko/20110920_Firefox/3.6.23 http_proxy_status_code=200 reason=- disposition=1034 policy=- role=8 duration=0 url=https://mail.google.com""", "${.app.name}", "websense"),
    (r"""<134>1 2018-03-21 17:25:25 MDS-72 CheckPoint 13752 - [action:"Update"; flags:"150784"; ifdir:"inbound"; logid:"160571424"; loguid:"{0x5ab27965,0x0,0x5b20a8c0,0x7d5707b6}"; origin:"192.168.32.91"; originsicname:"CN=GW91,O=Domain2_Server..cuggd3"; sequencenum:"1"; time:"1521645925"; version:"5"; auth_method:"Machine Authentication (Active Directory)"; auth_status:"Successful Login"; authentication_trial:"this is a reauthentication for session 9a026bba"; client_name:"Active Directory Query"; client_version:"R80.10"; domain_name:"spec.mgmt"; endpoint_ip:"192.168.32.69"; identity_src:"AD Query"; identity_type:"machine"; product:"Identity Awareness"; snid:"9a026bba"; src:"192.168.32.69"; src_machine_group:"All Machines"; src_machine_name:"yonatanad";]""", "${.app.name}", "checkpoint"),
    (r"""<134>1 2018-03-21T17:25:25 MDS-72 CheckPoint 13752 - [action:"Update"; flags:"150784"; ifdir:"inbound"; logid:"160571424"; loguid:"{0x5ab27965,0x0,0x5b20a8c0,0x7d5707b6}"; origin:"192.168.32.91"; originsicname:"CN=GW91,O=Domain2_Server..cuggd3"; sequencenum:"1"; time:"1521645925"; version:"5"; auth_method:"Machine Authentication (Active Directory)"; auth_status:"Successful Login"; authentication_trial:"this is a reauthentication for session 9a026bba"; client_name:"Active Directory Query"; client_version:"R80.10"; domain_name:"spec.mgmt"; endpoint_ip:"192.168.32.69"; identity_src:"AD Query"; identity_type:"machine"; product:"Identity Awareness"; snid:"9a026bba"; src:"192.168.32.69"; src_machine_group:"All Machines"; src_machine_name:"yonatanad";]""", "${.app.name}", "checkpoint"),
    (r"""<134>1 2019-11-27T02:58:25Z ABDP-CPLOG01 CheckPoint 22103 - [action:"Accept"; flags:"18692"; ifdir:"inbound"; ifname:"bond1.734"; loguid:"{0x5ddde651,0x48,0xca96040a,0xc000001b}"; origin:"10.4.150.2"; time:"1574823505"; version:"1"; __policy_id_tag:"product=VPN-1 & FireWall-1[db_tag={12D7A082-42F0-B240-8103-29F2F6FF139C};mgmt=ABDP-MGT01;date=1573204576;policy_name=Cutover_Final_V3_20161031]"; dst:"124.156.190.9"; nat_addtnl_rulenum:"1"; nat_rulenum:"8"; origin_sic_name:"CN=ABDP-CPFW01,O=ABDP-CPMGT01..94r78q"; product:"VPN-1 & FireWall-1"; proto:"6"; rule:"8"; rule_name:"3G IP Pool Outgoing"; rule_uid:"{DF64DC15-BEDF-4246-8B71-A75E0991C5D9}"; s_port:"64442"; service:"80"; service_id:"http"; src:"10.7.79.7"; xlatedport:"0"; xlatedst:"0.0.0.0"; xlatesport:"49254"; xlatesrc:"202.1.50.67"; ]""", "${.app.name}", "checkpoint"),
    (r"""time=1557767758|hostname=r80test|product=Firewall|layer_name=Network|layer_uuid=c0264a80-1832-4fce-8a90-d0849dc4ba33|match_id=1|parent_rule=0|rule_action=Accept|rule_uid=4420bdc0-19f3-4a3e-8954-03b742cd3aee|action=Accept|ifdir=inbound|ifname=eth0|logid=0|loguid={0x5cd9a64e,0x0,0x5060a8c0,0xc0000001}|origin=192.168.96.80|originsicname=cn\=cp_mgmt,o\=r80test..ymydp2|sequencenum=1|time=1557767758|version=5|dst=192.168.96.80|inzone=Internal|outzone=Local|proto=6|s_port=63945|service=443|service_id=https|src=192.168.96.27|""", "${.app.name}", "checkpoint"),
    (r'''<189>date=2021-01-15 time=12:58:59 devname="FORTI_111" devid="FG100D3G12801312" logid="0001000014" type="traffic" subtype="local" level="notice" vd="root" eventtime=1610704739683510055 tz="+0300" srcip=91.234.154.139 srcname="91.234.154.139" srcport=45295 srcintf="wan1" srcintfrole="wan" dstip=213.59.243.9 dstname="213.59.243.9" dstport=46730 dstintf="unknown0" dstintfrole="undefined" sessionid=2364413215 proto=17 action="deny" policyid=0 policytype="local-in-policy" service="udp/46730" dstcountry="Russian Federation" srccountry="Russian Federation" trandisp="noop" app="udp/46730" duration=0 sentbyte=0 rcvdbyte=0 sentpkt=0 appcat="unscanned" crscore=5 craction=262144 crlevel="low"''', "${.app.name}", "fortigate"),
]


@pytest.mark.parametrize(
    "input_message, template, expected_value", test_parameters_raw,
    ids=list(map(str, range(len(test_parameters_raw)))),
)
def test_application_raw(config, syslog_ng, input_message, template, expected_value):
    config.add_include("scl.conf")

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    app_parser = config.create_app_parser(topic="syslog-raw")

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template + '\n'))
    config.create_logpath(statements=[generator_source, app_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == expected_value



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2021 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

SYSLOG_NG_DEFAULT_PRI = "38"


@pytest.mark.parametrize(
    "pri_number, expected_pri", [
        ("'0'", "0"),
        ("'42'", "42"),
        ("'1023'", "1023"),
        ("'5789'", SYSLOG_NG_DEFAULT_PRI),
        ("'-2'", SYSLOG_NG_DEFAULT_PRI),
        ("'test'", SYSLOG_NG_DEFAULT_PRI),
        ("''", SYSLOG_NG_DEFAULT_PRI),
    ], ids=["min_value", "valid_value", "max_value", "invalid_big_number", "invalid_negative", "invalid_letters", "empty_value"],
)
def test_set_pri(config, syslog_ng, log_message, bsd_formatter, pri_number, expected_pri):
    input_message = bsd_formatter.format_message(log_message.priority(38))

    file_source = config.create_file_source(file_name="input.log")
    rewrite_pri = config.create_rewrite_set_pri(pri_number)
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$PRI\n"))
    config.create_logpath(statements=[file_source, rewrite_pri, file_destination])

    file_source.write_log(input_message)
    syslog_ng.start(config)

    assert file_destination.read_log().strip() == expected_pri



#############################################################################
# Copyright (c) 2007-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import os

def is_running_in_build_tree():
    return 'SYSLOG_NG_BINARY' not in os.environ

def get_module_path_from_binary():
    module_path = os.popen("%s --version | grep \"Module-Path:\" | cut -d ' ' -f 2" % get_syslog_ng_binary(), 'r').read().strip()
    return module_path

def get_config_version_from_binary():
    config_version = os.popen("%s --version | grep \"Config version:\" | cut -d ':' -f 2" % get_syslog_ng_binary(), 'r').read().strip()
    return config_version

def format_module_path_for_intree_modules():
    module_path = ''
    for (root, dirs, files) in os.walk(os.path.abspath(os.path.join(os.environ['top_builddir'], 'modules'))):
        module_path += ':'.join(map(lambda x: root + '/' + x + '/.libs', dirs))
        module_path += ':'
        module_path += ':'.join(map(lambda x: root + '/' + x, dirs))
        break
    return module_path

def get_module_path():
    if is_running_in_build_tree():
        module_path = format_module_path_for_intree_modules()
    else:
        module_path = get_module_path_from_binary()
    return module_path

def get_syslog_ng_binary():
    return os.getenv('SYSLOG_NG_BINARY', '../../syslog-ng/syslog-ng')

def is_premium():
    version = os.popen('%s -V' % get_syslog_ng_binary(), 'r').read()
    if version.find('premium-edition') != -1:
        return True
    return False

def has_module(module):
    avail_mods = os.popen('%s -V | grep ^Available-Modules: ' % get_syslog_ng_binary(), 'r').read()
    if avail_mods.find(module) != -1:
        return True
    return False


is_premium_edition = is_premium()
if is_premium_edition:
    logstore_store_supported = True
    wildcard_file_source_supported = True
else:
    logstore_store_supported = False
    wildcard_file_source_supported = True

port_number = os.getpid() % 30000 + 33000
ssl_port_number = port_number + 1
port_number_syslog = port_number + 2
port_number_network = port_number + 3

syslog_ng_version = "current"

current_dir = os.getcwd()
try:
    src_dir = os.environ["srcdir"]
except KeyError:
    src_dir = current_dir



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.common.blocking import wait_until_true

NUMBER_OF_MESSAGES = 10


def test_pp_with_simple_tcp_connection(config, port_allocator, syslog_ng, loggen):
    network_source = config.create_network_source(ip="localhost", port=port_allocator(), transport="proxied-tcp")
    file_destination = config.create_file_destination(file_name="output.log")
    config.create_logpath(statements=[network_source, file_destination])

    syslog_ng.start(config)

    loggen.start(network_source.options["ip"], network_source.options["port"], inet=True, stream=True, number=NUMBER_OF_MESSAGES)
    wait_until_true(lambda: loggen.get_sent_message_count() == NUMBER_OF_MESSAGES)

    # We could check the source side stats, too, but it is not yet implemented
    assert not file_destination.get_path().exists()



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from ..lexertoken import LexerToken
from ..tablexer import TabLexer
from ..lexer import Lexer
from .test_lexer import TestLexer


class GenNumbersLexer(Lexer):
    def __init__(self, num_tokens, mark_last_partial=False):
        self._token_count = 0
        self._num_tokens = num_tokens
        self._mark_last_partial = mark_last_partial
        self._current_position = 0

    def token(self):
        if self._token_count < self._num_tokens:
            lexpos = self._current_position
            self._token_count += 1
            self._current_position = self._token_count * 2

            return LexerToken(type="NUMBER",
                              value=str(self._token_count),
                              partial=self._mark_last_partial and self._token_count == self._num_tokens,
                              lexpos=lexpos)
        return None

    def input(self, text):
        self._current_position = 0
        self._token_count = 0

    def get_position(self):
        return self._current_position


class TestTabLexer(TestLexer):
    def setUp(self):
        self._lexer = self._construct_lexer()

    # pylint: disable=arguments-differ
    def _construct_lexer(self, num_tokens=5, mark_last_partial=False):
        lexer = TabLexer(GenNumbersLexer(num_tokens, mark_last_partial=mark_last_partial))
        self._consume_all_tokens(lexer)
        lexer.input('')
        return lexer

    @staticmethod
    def _consume_all_tokens(lexer):
        while lexer.token() is not None:
            pass

    def test_tab_lexer_token_returns_none_if_the_underlying_lexer_is_empty(self):
        self._lexer = self._construct_lexer(0)
        self._assert_next_token_is_tab(tab_position=0)
        self.assertEqual(self._lexer.token(), None)

    def test_tab_lexer_token_returns_token_if_the_underlying_lexer_has_elements(self):
        self._lexer = self._construct_lexer(2)
        self.assertEqual(self._lexer.token().value, '1')
        self.assertEqual(self._lexer.token().value, '2')
        self._assert_next_token_is_tab(tab_position=4)
        self.assertIsNone(self._lexer.token())

    def test_partial_is_replaced_by_tab(self):
        self._lexer = self._construct_lexer(2, mark_last_partial=True)
        self.assertEqual(self._lexer.token().value, '1')
        self._assert_next_token_is_tab(tab_position=2)
        self.assertIsNone(self._lexer.token())
        self.assertEqual(self._lexer.get_replaced_token().value, '2')

    def test_tab_is_appended_if_no_partial_token(self):
        self._lexer = self._construct_lexer(2, mark_last_partial=False)
        self._lexer.set_drop_last_token(False)
        self.assertEqual(self._lexer.token().value, '1')
        self.assertEqual(self._lexer.token().value, '2')
        self._assert_next_token_is_tab(tab_position=4)
        self.assertIsNone(self._lexer.token())
        self.assertIsNone(self._lexer.get_replaced_token())

    def test_last_token_is_replaced_by_a_tab_if_theres_no_partial_and_after_last_insertion_is_not_requested(self):
        self._lexer = self._construct_lexer(2, mark_last_partial=False)
        self._lexer.set_drop_last_token(True)
        self.assertEqual(self._lexer.token().value, '1')
        self._assert_next_token_is_tab(tab_position=2)
        self.assertIsNone(self._lexer.token())
        self.assertEqual(self._lexer.get_replaced_token().value, '2')

    def test_tab_lexer_returns_tab_token_at_the_end_of_the_sequence(self):
        self._lexer = self._construct_lexer(2)
        self._assert_n_numbered_tokens_available(2)
        self._assert_next_token_is_tab(tab_position=4)
        self.assertIsNone(self._lexer.token())

    def test_tab_lexer_can_replace_last_token_to_a_tab(self):
        self._lexer = self._construct_lexer(0)
        self._lexer.set_drop_last_token(True)
        self._assert_next_token_is_tab(tab_position=0)
        self.assertIsNone(self._lexer.token())

        self._lexer = self._construct_lexer(1)
        self._lexer.set_drop_last_token(True)
        self._assert_next_token_is_tab(tab_position=0)
        self.assertIsNone(self._lexer.token())

        self._lexer = self._construct_lexer(2)
        self._lexer.set_drop_last_token(True)
        self._assert_n_numbered_tokens_available(1)
        self._assert_next_token_is_tab(tab_position=2)
        self.assertIsNone(self._lexer.token())

        self._lexer = self._construct_lexer(3)
        self._lexer.set_drop_last_token(True)
        self._assert_n_numbered_tokens_available(2)
        self._assert_next_token_is_tab(tab_position=4)
        self.assertIsNone(self._lexer.token())

    def _assert_n_numbered_tokens_available(self, n):
        for i in range(1, n + 1):
            self.assertEqual(self._lexer.token().value, str(i))

    def _assert_current_token_position_equals(self, lexpos):
        self.assertEqual(self._current_token.lexpos, lexpos)

    def _assert_next_token_is_tab(self, tab_position=None):
        self._next_token()
        self._assert_current_token_type_equals("TAB")
        if tab_position is not None:
            self._assert_current_token_position_equals(tab_position)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.message_builder.log_message import LogMessage


def write_msg_with_fields(file_source, bsd_formatter, hostname):
    log_message = LogMessage().hostname(hostname)
    input_message = bsd_formatter.format_message(log_message)
    expected_message = bsd_formatter.format_message(log_message.remove_priority())
    file_source.write_log(input_message)
    return expected_message


def test_flags_final(config, syslog_ng, bsd_formatter):
    # Check the correct output if the logpath is the following
    # log {
    #     source(s_file);
    #     log { filter{host("host-A")}; destination(d_file1); flags(final); };
    #     log { destination(d_file2); };
    # };
    # input logs:
    # Oct 11 22:14:15 host-A testprogram: message from host-A
    # Oct 11 22:14:15 host-B testprogram: message from host-B

    config.update_global_options(keep_hostname="yes")

    file_source = config.create_file_source(file_name="input.log")
    host_filter = config.create_filter(host="'host-A'")
    file_destination1 = config.create_file_destination(file_name="output1.log")
    file_destination2 = config.create_file_destination(file_name="output2.log")

    inner_logpath1 = config.create_inner_logpath(statements=[host_filter, file_destination1], flags="final")
    inner_logpath2 = config.create_inner_logpath(statements=[file_destination2])

    config.create_logpath(statements=[file_source, inner_logpath1, inner_logpath2])

    expected_message1 = write_msg_with_fields(file_source, bsd_formatter, "host-A")
    expected_message2 = write_msg_with_fields(file_source, bsd_formatter, "host-B")

    syslog_ng.start(config)

    dest1_logs = file_destination1.read_log()
    # host("host-A") matches on first message
    assert expected_message1 in dest1_logs

    dest2_logs = file_destination2.read_log()
    # second message should arrived only into destination3
    # first message was stopped on first logpath(), because of
    # flags(final)
    assert expected_message2 in dest2_logs



#############################################################################
# Copyright (c) 2007-2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *

config = """@version: %(syslog_ng_version)s

options { ts_format(iso); chain_hostnames(no); keep_hostname(yes); threaded(yes); };

source s_int { internal(); };
source s_unix { unix-stream("log-stream" flags(expect-hostname)); };

filter f_facility { message("facility"); };
filter f_facility1 { facility(syslog); };
filter f_facility2 { facility(kern); };
filter f_facility3 { facility(mail); };
filter f_facility4 { facility(daemon,auth,lpr); };

destination d_facility1 { file("test-facility1.log"); logstore("test-facility1.lgs"); };
destination d_facility2 { file("test-facility2.log"); logstore("test-facility2.lgs"); };
destination d_facility3 { file("test-facility3.log"); logstore("test-facility3.lgs"); };
destination d_facility4 { file("test-facility4.log"); logstore("test-facility4.lgs"); };

log { source(s_unix);
    filter(f_facility);
    log { filter(f_facility1); destination(d_facility1); };
    log { filter(f_facility2); destination(d_facility2); };
    log { filter(f_facility3); destination(d_facility3); };
    log { filter(f_facility4); destination(d_facility4); };
};

filter f_level { message("level"); };
filter f_level1 { level(debug); };
filter f_level2 { level(info); };
filter f_level3 { level(notice); };
filter f_level4 { level(warning..crit); };

destination d_level1 { file("test-level1.log"); logstore("test-level1.lgs"); };
destination d_level2 { file("test-level2.log"); logstore("test-level2.lgs"); };
destination d_level3 { file("test-level3.log"); logstore("test-level3.lgs"); };
destination d_level4 { file("test-level4.log"); logstore("test-level4.lgs"); };

log { source(s_unix);
    filter(f_level);
    log { filter(f_level1); destination(d_level1); };
    log { filter(f_level2); destination(d_level2); };
    log { filter(f_level3); destination(d_level3); };
    log { filter(f_level4); destination(d_level4); };
};

""" % locals()

def test_facility_single():
    messages = (
      (41, 'facility1'),
      (1, 'facility2'),
      (17, 'facility3'),
    )
    expected = [None,] * len(messages)

    s = SocketSender(AF_UNIX, 'log-stream', dgram=0, repeat=10)
    for ndx in range(0, len(messages)):
        if not expected[ndx]:
            expected[ndx] = []
        expected[ndx].extend(s.sendMessages(messages[ndx][1], pri=messages[ndx][0]))

    for ndx in range(0, len(messages)):
        if not check_file_expected('test-facility%d' % (ndx + 1,), expected[ndx]):
            return False
    return True

def test_facility_multi():
    messages = (
      (25, 'facility4'),
      (33, 'facility4'),
      (49, 'facility4'),
    )
    expected = []

    s = SocketSender(AF_UNIX, 'log-stream', dgram=0, repeat=10)
    for ndx in range(0, len(messages)):
        expected.extend(s.sendMessages(messages[ndx][1], pri=messages[ndx][0]))

    if not check_file_expected('test-facility4', expected):
        return False
    return True


def test_level_single():
    messages = (
      (7, 'level1'),
      (6, 'level2'),
      (5, 'level3'),
    )
    expected = [None,] * len(messages)

    s = SocketSender(AF_UNIX, 'log-stream', dgram=0, repeat=10)
    for ndx in range(0, len(messages)):
        if not expected[ndx]:
            expected[ndx] = []
        expected[ndx].extend(s.sendMessages(messages[ndx][1], pri=messages[ndx][0]))

    for ndx in range(0, len(messages)):
        if not check_file_expected('test-level%d' % (ndx + 1,), expected[ndx]):
            return False
    return True

def test_level_multi():
    messages = (
      (4, 'level4'),
      (3, 'level4'),
      (2, 'level4'),
    )
    expected = []

    s = SocketSender(AF_UNIX, 'log-stream', dgram=0, repeat=10)
    for ndx in range(0, len(messages)):
        expected.extend(s.sendMessages(messages[ndx][1], pri=messages[ndx][0]))

    if not check_file_expected('test-level4', expected):
        return False
    return True



#!/usr/bin/env python3
import json, sys

j = None

if len(sys.argv) == 1:
    j = json.loads(sys.stdin.read())
else:
    file_name = sys.argv[1]
    with open(file_name) as f:
        j = json.loads(f.read())

nodes = j["nodes"]
arcs = j["arcs"]

print("digraph D {")
for node in nodes:
    print("  Node{} [label=\"{} {}\", shape=box]".format(node["node"], node["node"], ", ".join(node["info"])))
for arc in arcs:
    print("  Node{} -> Node{} [label=\"{}\"]".format(arc["from"], arc["to"], arc["type"]))
print("}")



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from .test_completer import CompleterTestCase
from ..debuggercli import DebuggerCLI


class TestDebuggerCLI(CompleterTestCase):
    def _construct_completer(self):
        return DebuggerCLI().get_root_completer()

    def test_no_text_offers_the_potential_prefixes_of_a_template_expr(self):
        self._assert_completions_offered(entire_input="print ", word="", expected_completions=['$', '$(', '${'])

    def test_literal_text_offers_potential_prefixes_of_a_template_expr(self):
        self._assert_completions_offered(entire_input="print 'foobar ", word="", expected_completions=['$', '$(', '${'])

    def test_dollar_offers_possible_macro_completions(self):
        self._assert_completions_offered(entire_input="print $", word="$",
                                         expected_completions=['$MSG', '$MSGHDR', '$DATE', '${', '$('])

    def test_dollar_brace_offers_possible_macro_completions(self):
        self._assert_completions_offered(entire_input="print ${", word="${",
                                         expected_completions=["${MSG}", "${DATE}"])

    def test_dollar_paren_offers_template_functions(self):
        self._assert_completions_offered(entire_input="print $(", word="$(",
                                         expected_completions=["$(echo "])

    def test_literal_text_plus_dollar_offers_macros_using_the_literal_as_prefix(self):
        self._assert_completions_offered(entire_input="print foobar$", word="foobar$",
                                         expected_completions=["foobar$("])

    def test_literal_text_plus_dollar_paren_offers_template_functions_using_the_literal_as_prefix(self):
        self._assert_completions_offered(entire_input="print foobar$(", word="foobar$(",
                                         expected_completions=["foobar$(echo ", 'foobar$(format-json '])

    def test_literal_text_plus_dollar_brace_offers_macros_using_the_literal_as_prefix(self):
        self._assert_completions_offered(entire_input="print foobar${", word="foobar${",
                                         expected_completions=["foobar${MSG}"])

    def test_template_function_offers_potential_opts(self):
        self._assert_completions_offered(entire_input="print 'foobar$(format-json ", word="",
                                         expected_completions=['--pair '])
        self._assert_completions_offered(entire_input="print 'foobar$(echo ", word="",
                                         expected_completions=['$', '$(', '${'])

    def test_complex_cases(self):
        self._assert_completions_offered(entire_input="print 'foo$(format-json --key *)bar", word="",
                                         expected_completions=['$(', '${', '$'])
        self._assert_completions_offered(entire_input="print 'foo$(format-json --key *)bar$(echo ", word="",
                                         expected_completions=['$(', '${', '$'])
        self._assert_completions_offered(entire_input="print 'foo$(format-json --key *)${MSG}$(echo ", word="",
                                         expected_completions=['$(', '${', '$'])
        self._assert_completions_offered(entire_input="print 'foo$(format-json --key *)${MSG}$(echo $MSGHDR)", word="",
                                         expected_completions=['$(', '${', '$'])



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from .test_lexer import TestLexer
from .. import commandlinelexer


class TestCommandLineLexer(TestLexer):

    def _construct_lexer(self):
        return commandlinelexer.CommandLineLexer()

    def test_lexer_returns_none_for_empty_string(self):
        self._lexer.input("")
        self._assert_next_token_is_none()

    def test_single_quote_character_is_returned_as_a_partial_token(self):
        for quote in ('"', "'"):
            self._lexer.input(quote)
            self._assert_next_token_is_partial()

    def test_pair_of_quotes_is_returned_as_an_empty_string(self):
        self._lexer.input("''")
        self._next_token()
        self._assert_current_token_is_not_partial()
        self._assert_current_token_value_equals('')

    def test_quoted_character_is_returned_as_the_character(self):
        self._lexer.input("'a'")
        self._next_token()
        self._assert_current_token_is_not_partial()
        self._assert_current_token_value_equals('a')

    def test_quoted_string_is_returned_as_the_string(self):
        self._lexer.input('"foo bar"')
        self._assert_next_token_value_equals('foo bar')

    def test_unquoted_string_is_returned_as_the_string(self):
        self._lexer.input('foo')
        self._assert_next_token_value_equals('foo')

    def test_unquoted_prefix_and_then_a_quoted_string_is_concatenated(self):
        self._lexer.input('foo"bar"')
        self._assert_next_token_value_equals('foobar')

    def test_double_quotes_allow_backslash_as_a_single_char_escape(self):
        self._lexer.input(r'"foo\""')
        self._assert_next_token_value_equals('foo"')

    def test_white_space_separates_tokens(self):
        self._lexer.input("""'foo'  "bar"  baz  """)
        self._assert_next_token_value_equals('foo')
        self._assert_next_token_value_equals('bar')
        self._assert_next_token_value_equals('baz')
        self._assert_next_token_is_none()

    def test_unclosed_string_is_returned_as_a_partial_token(self):
        self._lexer.input("""'foo' "bar" 'baz""")
        self._next_token()
        self._assert_current_token_value_equals('foo')
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals('bar')
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals('baz')
        self._assert_current_token_is_partial()

    def test_single_opened_paren_is_returned_as_a_partial_token(self):
        self._lexer.input("(")
        self._next_token()
        self._assert_current_token_value_equals('(')
        self._assert_current_token_is_partial()

    def test_pair_of_parens_is_returned_as_a_pair_of_parens(self):
        self._lexer.input("()")

        self._next_token()
        self._assert_current_token_value_equals('()')
        self._assert_current_token_is_not_partial()

    def test_token_enclosed_in_parens_is_returned_as_a_token_in_parens(self):
        self._lexer.input("(foo)")
        self._next_token()
        self._assert_current_token_value_equals('(foo)')
        self._assert_current_token_is_not_partial()

    def test_whitespace_in_parens_doesnt_terminate_the_token(self):
        self._lexer.input("(foo bar)")
        self._next_token()
        self._assert_current_token_value_equals('(foo bar)')
        self._assert_current_token_is_not_partial()

    def test_closing_paren_terminates_the_token_only_if_properly_paired(self):
        self._lexer.input("(foo bar (baz bax)) next-token")
        self._next_token()
        self._assert_current_token_value_equals('(foo bar (baz bax))')
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals('next-token')
        self._assert_current_token_is_not_partial()

    def test_one_too_many_closing_parens_is_interpreted_as_a_separate_token(self):
        self._lexer.input("(foo bar )) next-token")

        self._next_token()
        self._assert_current_token_value_equals('(foo bar )')
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals(')')
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals('next-token')
        self._assert_current_token_is_not_partial()

    def test_quotes_within_parens_are_left_intact_so_a_recursive_parsing_will_find_them(self):
        self._lexer.input("(foo 'bar baz') next-token")
        self._next_token()
        self._assert_current_token_value_equals("(foo 'bar baz')")
        self._assert_current_token_is_not_partial()

    def test_parens_in_quotes_are_not_counted_against_the_paren_balance(self):
        self._lexer.input("(foo 'bar )')    next-token")
        self._next_token()
        self._assert_current_token_value_equals("(foo 'bar )')")
        self._assert_current_token_is_not_partial()

        self._next_token()
        self._assert_current_token_value_equals("next-token")
        self._assert_current_token_is_not_partial()



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from .test_completer import CompleterTestCase
from ..macrocompleter import MacroCompleter


class TestMacroCompleter(CompleterTestCase):
    # pylint: disable=arguments-differ
    def _construct_completer(self, macros=None):
        return MacroCompleter(macros=macros or ['MSG', 'MSGHDR', 'HOST',
                                                '.unix.uid', '.unix.gid',
                                                'S_DATE', 'R_MONTH', 'C_WEEK', 'DATE',
                                                'smallcaps',
                                                '0', '1', '2', '10', '100'])

    def test_only_a_dollar_is_offered_for_an_empty_string(self):
        self._assert_completions_offered('', ('$', '${'))

    def test_dollar_brace_is_offered_for_a_dollar(self):
        self._assert_completion_offered('$', '${')

    def test_unqualified_macros_are_offered_for_a_dollar(self):
        self._assert_completions_offered('$', ('$MSG', '$HOST'))

    def test_qualified_macros_are_offered_with_a_brace_for_a_dollar(self):
        self._assert_completion_offered('$', '${.unix.uid}')
        self._assert_completion_not_offered('$', '$.unix.uid')

    def test_small_numbered_matches_are_offered_for_a_dollar(self):
        # small means < 10
        self._assert_completion_offered('$', '$0')

    def test_large_numbered_matches_are_not_offered_for_a_dollar(self):
        # large means > 10
        self._assert_completions_not_offered('$', ('$10', '$100'))

    def test_all_numbered_matches_are_offered_for_dollar_plus_digit(self):
        # ${1 ... lists all numbered macros
        self._assert_completions_offered('$1', ('$1', '$10', '$100'))

    def test_all_macros_except_large_numbered_matches_are_offered_for_a_dollar_brace(self):
        # ${ }
        self._assert_completions_offered('${', ('${HOST}', '${MSG}', '${.unix.uid}', '${0}', '${1}'))
        self._assert_completions_not_offered('${', ('${10}', '${100}'))

    def test_all_numbered_matches_are_offered_for_dollar_brace_digit(self):
        self._assert_completions_offered('${1', ('${1}', '${10}', '${100}'))

    def test_date_macros_are_not_offered_for_a_dollar(self):
        self._assert_completions_not_offered('$', ('$S_DATE', '$R_MONTH'))

    def test_date_shortcuts_are_offered_for_a_dollar(self):
        self._assert_completions_offered('$', ['$R_* (received time)',
                                               '$S_* (stamp in message)',
                                               '$C_* (current time)'])

    def test_date_shortcuts_are_offered_for_a_dollar_brace(self):
        self._assert_completions_offered('${', ['${R_*} (received time)',
                                                '${S_*} (stamp in message)',
                                                '${C_*} (current time)'])

    def test_date_macros_are_offered_if_the_prefix_is_correct(self):
        self._assert_completions_offered('$R_', ['$R_MONTH'])
        self._assert_completions_offered('${R_', ['${R_MONTH}'])
        self._assert_completions_offered('$S_', ['$S_DATE'])
        self._assert_completions_offered('${S_', ['${S_DATE}'])
        self._assert_completions_offered('$C_', ['$C_WEEK'])
        self._assert_completions_offered('${C_', ['${C_WEEK}'])

    def test_date_macros_are_not_offered_if_the_prefix_doesnot_match(self):
        self._assert_completion_not_offered('$R', '$R_MONTH')
        self._assert_completion_not_offered('$S', '$S_DATE')
        self._assert_completion_not_offered('$C', '$C_DATE')

    def test_date_wildcards_are_not_offered_if_the_prefix_is_correct(self):
        self._assert_completions_not_offered('$R_', '$R_* (received time)')
        self._assert_completions_not_offered('${R_', '${R_*} (received time)')
        self._assert_completions_not_offered('$S_', '$S_* (stamp in message)')
        self._assert_completions_not_offered('${S_}', '${S_*} (stamp in message)')
        self._assert_completions_not_offered('$C_', '$C_* (current time)')
        self._assert_completions_not_offered('${C_', '${C_*} (current time)')

    def test_prefix_is_as_good_as_the(self):
        # $ + number as prefix
        self._assert_completions_offered('$10', ('$10', '$100'))
        # $ + macro first character
        self._assert_completions_offered('$M', ('$MSGHDR', '$MSG'))
        # $ + macro first character (small caps)
        self._assert_completions_offered('$s', ('$smallcaps',))
        # $ + brace + number
        self._assert_completions_offered('${10', ('${10}', '${100}'))
        # $ + brace + unqualified macro first character
        self._assert_completions_offered('${M', ('${MSGHDR}', '${MSG}'))
        # $ + brace + qualified macro first character
        self._assert_completions_offered('${.', ('${.unix.uid}', '${.unix.gid}'))

    def test_only_completions_that_start_with_word_are_listed_as_completions(self):
        for word in ('$', '$1', '$M'):
            self._assert_completions_start_with_word(word)

    def test_nothing_is_offered_if_the_entire_input_diverged_already(self):
        completions = self._get_completions('', entire_input='$(echo ')
        self.assertEqual(completions, [])
        completions = self._get_completions('', entire_input='almafa ')
        self.assertEqual(completions, [])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_config.renderer import render_statement


# noqa: E122


def render_filterx_exprs(expressions):
    return '\n'.join(f"filterx {{ {expr} }};" for expr in expressions)


def create_config(config, init_exprs, true_exprs=(), false_exprs=(), final_exprs=(), msg="foobar"):
    file_true = config.create_file_destination(file_name="dest-true.log", template="'$MSG\n'")
    file_false = config.create_file_destination(file_name="dest-false.log", template="'$MSG\n'")
    file_final = config.create_file_destination(file_name="dest-final.log", template="'$MSG\n'")

    preamble = f"""
@version: {config.get_version()}

options {{ stats(level(1)); }};

source genmsg {{
    example-msg-generator(
        num(1)
        template("{msg}")
        values(
            "values.str" => string("string"),
            "values.bool" => boolean(true),
            "values.int" => int(5),
            "values.double" => double(32.5),
            "values.datetime" => datetime("1701350398.123000+01:00"),
            "values.list" => list("foo,bar,baz"),
            "values.null" => null(""),
            "values.bytes" => bytes("binary whatever"),
            "values.protobuf" => protobuf("this is not a valid protobuf!!"),
            "values.json" => json('{{"emb_key1": "emb_key1 value", "emb_key2": "emb_key2 value"}}'),
            "values.true_string" => string("boolean:true"),
            "values.false_string" => string("boolean:false"),
        )
    );
}};

destination dest_true {{
    {render_statement(file_true)};
}};

destination dest_false {{
    {render_statement(file_false)};
}};

destination dest_final {{
    {render_statement(file_final)};
}};

log {{
    source(genmsg);
    {render_filterx_exprs(init_exprs)};
    if {{
        {render_filterx_exprs(true_exprs)}
        destination(dest_true);
    }} else {{
        {render_filterx_exprs(false_exprs)}
        destination(dest_false);
    }};
    {render_filterx_exprs(final_exprs)}
    destination(dest_final);
}};
"""
    config.set_raw_config(preamble)
    return (file_true, file_false, file_final)


def test_message_tied_variables_are_propagated_to_the_output(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, [
            """
                $foo = "kecske";
                isset($foo);
            """,
            """
                isset($foo);
                $MSG = $foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "kecske\n"


def test_message_tied_variables_in_braces_are_propagated_to_the_output(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, [
            """
                ${.foo.bar.baz} = "kecske";
                isset(${.foo.bar.baz});
            """,
            """
                isset(${.foo.bar.baz});
                $MSG = ${.foo.bar.baz};
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "kecske\n"


def test_message_tied_variables_are_propagated_to_the_output_in_junctions(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, init_exprs=[
            """
                $foo = "kecske";
                isset($foo);
            """,
        ], true_exprs=[
            """
            isset($foo);
            $MSG = $foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "kecske\n"


def test_message_tied_variables_do_not_propagate_to_parallel_branches(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, init_exprs=[
            """
            $foo = "kecske";
            isset($foo);
            """,
        ], true_exprs=[
            """
            isset($foo);
            $bar = $foo;
            isset($bar);
            $foo = "not kecske";
            false;
            """,
        ], false_exprs=[
            """
            isset($foo);
            not isset($bar);
            $MSG = $foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_false.get_stats()["processed"] == 1
    assert "processed" not in file_true.get_stats()
    assert file_false.read_log() == "kecske\n"


def test_floating_variables_are_dropped_at_the_end_of_the_scope(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, [
            """
            foo = "kecske";
            isset(foo);
            """,
            """
            not isset(foo);
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "foobar\n"


def test_floating_variables_are_dropped_at_the_end_of_the_scope_but_can_be_recreated(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, [
            """
            foo = "kecske";
            isset(foo);
            """,
            """
            not isset(foo);
            foo = "barka";
            isset(foo);
            $MSG = foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "barka\n"


def test_declared_variables_are_retained_across_scopes(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, [
            """
            declare foo = "kecske";
            isset(foo);
            """,
            """
            isset(foo);
            foo = "barka";
            """,
            """
            isset(foo);
            $MSG = foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "barka\n"


def test_declared_variables_are_retained_across_scopes_and_junctions(config, syslog_ng):
    (file_true, file_false, _) = create_config(
        config, init_exprs=[
            """
            declare foo = "kecske";
            isset(foo);
            """,
        ], true_exprs=[
            """
            isset(foo);
            foo = "barka";
            """,
            """
            isset(foo);
            $MSG = foo;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "barka\n"


def test_changes_in_abandoned_branches_are_ignored(config, syslog_ng):
    (file_true, file_false, file_final) = create_config(
        config, init_exprs=[
            """
                $json = json({"common": "common"});
                $variable = "something";
                unset($variable);
            """,
        ], true_exprs=[
            """
                $json += {"iftrue": "true"};
                $json;
                false;
            """,
        ], false_exprs=[
            """
                $json += {"iffalse": "false"};
                $json;
            """,
        ], final_exprs=[
            """
                $MSG = $json;
            """,
        ],
    )
    syslog_ng.start(config)

    assert file_false.get_stats()["processed"] == 1
    assert "processed" not in file_true.get_stats()
    assert file_false.read_log() == "foobar\n"

    assert file_final.get_stats()["processed"] == 1
    assert file_final.read_log() == '{"common":"common","iffalse":"false"}\n'



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import logging
from pathlib import Path
from subprocess import run
from typing import Dict, List, TextIO, Optional

logger = logging.getLogger("utils")
logger.setLevel(logging.INFO)


def execute_command(
    command: List[str],
    dir: Optional[Path] = None,
    env: Optional[Dict[str, str]] = None,
    input: Optional[str] = None,
    stdout: Optional[TextIO] = None,
) -> None:
    log_extras = {k: str(v) for k, v in filter(lambda kv: kv[1] is not None and kv[0] != "input", locals().items())}
    logger.info("Executing command.\t{}".format(log_extras))

    rc = run(
        command,
        input=bytes(input, "utf-8") if input is not None else None,
        stdout=stdout,
        cwd=dir,
        env=env,
    ).returncode
    if rc != 0:
        raise ChildProcessError("`{}` failed. dir={} env={} rc={}.".format(" ".join(command), dir, env, rc))


def move_file_without_overwrite(src: Path, dst: Path) -> None:
    if dst.exists():
        raise FileExistsError("Refusing to move file, destination already exists. src={} dst={}".format(src, dst))
    src.rename(dst)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from src.common.blocking import wait_until_true
from src.common.file import copy_shared_file
from src.common.file import File
from src.common.random_id import get_unique_id


def _test_pp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters, transport, input_messages, number_of_messages, expected_messages, template=None, password=None):
    server_key_path = copy_shared_file(testcase_parameters, "server.key")
    server_cert_path = copy_shared_file(testcase_parameters, "server.crt")
    output_file = "output.log"
    use_ssl = True if "tls" in transport else None
    use_inet = None if use_ssl else True
    use_passthrough = True if "passthrough" in transport else None

    if (use_ssl):
        syslog_source = config.create_syslog_source(
            ip="localhost",
            port=port_allocator(),
            transport=transport,
            flags="no-parse",
            tls={
                "key-file": server_key_path,
                "cert-file": server_cert_path,
                "peer-verify": '"optional-untrusted"',
            },
        )
    else:
        syslog_source = config.create_syslog_source(
            ip="localhost",
            port=port_allocator(),
            transport=transport,
            flags="no-parse",
        )
    if template:
        file_destination = config.create_file_destination(file_name=output_file, template=template)
    else:
        file_destination = config.create_file_destination(file_name=output_file)
    config.create_logpath(statements=[syslog_source, file_destination])

    syslog_ng.start(config)
    if password is not None:
        syslog_ng_ctl.credentials_add(credential=server_key_path, secret=password)

    loggen_input_file_path = Path("loggen_input_{}.txt".format(get_unique_id()))
    loggen_input_file = File(loggen_input_file_path)
    loggen_input_file.write_content_and_close(input_messages)
    loggen.start(
        syslog_source.options["ip"], syslog_source.options["port"],
        number=number_of_messages,
        dont_parse=True,
        read_file=str(loggen_input_file_path),
        syslog_proto=True,
        inet=use_inet,
        use_ssl=use_ssl,
        proxied=1,
        proxied_tls_passthrough=use_passthrough,
        proxy_src_ip="1.1.1.1", proxy_dst_ip="2.2.2.2", proxy_src_port="3333", proxy_dst_port="4444",
    )

    # seems proxy header is counted in get_sent_message_count(), so we need '-1'
    wait_until_true(lambda: loggen.get_sent_message_count() == number_of_messages - 1)

    assert file_destination.read_log() == expected_messages


def test_pp_syslog_tcp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters):
    TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
    INPUT_MESSAGES = "53 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\r\n"
    EXPECTED_MESSAGES = "1.1.1.1 2.2.2.2 3333 4444 4 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\n"
    NUMBER_OF_MESSAGES = 2
    _test_pp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters, '"proxied-tcp"', INPUT_MESSAGES, NUMBER_OF_MESSAGES, EXPECTED_MESSAGES, TEMPLATE)


def test_pp_syslog_tls(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters):
    TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
    INPUT_MESSAGES = "53 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\r\n"
    EXPECTED_MESSAGES = "1.1.1.1 2.2.2.2 3333 4444 4 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\n"
    NUMBER_OF_MESSAGES = 2
    _test_pp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters, '"proxied-tls"', INPUT_MESSAGES, NUMBER_OF_MESSAGES, EXPECTED_MESSAGES, TEMPLATE)


def test_pp_syslog_tls_with_passphrase(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters):
    TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
    INPUT_MESSAGES = "53 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\r\n"
    EXPECTED_MESSAGES = "1.1.1.1 2.2.2.2 3333 4444 4 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\n"
    NUMBER_OF_MESSAGES = 2
    _test_pp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters, '"proxied-tls"', INPUT_MESSAGES, NUMBER_OF_MESSAGES, EXPECTED_MESSAGES, TEMPLATE, password="asdfg")


def test_pp_syslog_tls_passthrough(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters):
    TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
    INPUT_MESSAGES = "53 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\r\n"
    EXPECTED_MESSAGES = "1.1.1.1 2.2.2.2 3333 4444 4 <2>Oct 11 22:14:15 myhostname sshd[1234]: message 0\n"
    NUMBER_OF_MESSAGES = 2
    _test_pp(config, syslog_ng, syslog_ng_ctl, port_allocator, loggen, testcase_parameters, '"proxied-tls-passthrough"', INPUT_MESSAGES, NUMBER_OF_MESSAGES, EXPECTED_MESSAGES, TEMPLATE)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2019 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest


test_parameters_syslog = [
    ("""<0>1 2012-03-05T15:10:34+02:00 localhost kernel 1234 - - transport_info='UDP: [10.30.35.100]:60513->[10.30.0.35]:162', DISMAN-EVENT-MIB::sysUpTimeInstance='(277195) 0:46:11.95', SNMPv2-MIB::snmpTrapOID.0='SNMPv2-SMI::enterprises.9.9.41.2.0.1.0', SNMPv2-SMI::enterprises.9.9.41.1.2.3.1.2.0='user', SNMPv2-SMI::enterprises.9.9.41.1.2.3.1.3.0='6', SNMPv2-SMI::enterprises.9.9.41.1.2.3.1.4.0='', SNMPv2-SMI::enterprises.9.9.41.1.2.3.1.5.0='[ 2771.344837] PF: filter/input DROP IN=eth0 OUT= MAC=08:00:27:d5:33:1e:34:e6:d7:1f:04:58:08:00 SRC=10.30.0.35 DST=10.30.35.100 LEN=60 TOS=0x00 PREC=0x00 TTL=1 ID=62309 DF PROTO=TCP SPT=52988 DPT=5355 WINDOW=29200 RES=0x00 SYN URGP=0 ', SNMPv2-SMI::enterprises.9.9.41.1.2.3.1.6.0='(277195) 0:46:11.95', SNMP-COMMUNITY-MIB::snmpTrapAddress.0='127.0.0.1'""", "${.app.name}", "iptables"),
    ("""<14>1 2019-04-12T14:29:06.576Z - RT_FLOW - RT_FLOW_SESSION_CLOSE [junos@2636.1.1.1.2.129 reason="TCP FIN" source-address="10.3.255.203" source-port="47776" destination-address="8.23.224.110" destination-port="80" connection-tag="0" service-name="junos-http" nat-source-address="10.3.136.49" nat-source-port="19162" nat-destination-address="8.23.224.110" nat-destination-port="80" nat-connection-tag="0" src-nat-rule-type="source rule" src-nat-rule-name="nat1" dst-nat-rule-type="N/A" dst-nat-rule-name="N/A" protocol-id="6" policy-name="permit_all" source-zone-name="trust" destination-zone-name="untrust" session-id-32="5" packets-from-client="6" bytes-from-client="337" packets-from-server="4" bytes-from-server="535" elapsed-time="1" application="HTTP" nested-application="UNKNOWN" username="N/A" roles="N/A" packet-incoming-interface="ge-0/0/0.0" encrypted="No" application-category="Web" application-sub-category="N/A" application-risk="4" application-characteristics="Can Leak Information;Supports File Transfer;Prone to Misuse;Known Vulnerabilities;Carrier of Malware;Capable of Tunneling;"] session closed TCP FIN: 10.3.255.203/47776->8.23.224.110/80 0x0 junos-http 10.3.136.49/19162->8.23.224.110/80 0x0 source rule nat1 N/A N/A 6 permit_all trust untrust 5 6(337) 4(535) 1 HTTP UNKNOWN N/A(N/A) ge-0/0/0.0 No Web N/A 4 Can Leak Information;Supports File Transfer;Prone to Misuse;Known Vulnerabilities;Carrier of Malware;Capable of Tunneling;""", "${.app.name}", "junos"),
    ("""Apr 18 12:33:50 nobody sudo[25173]:   nobody : TTY=pts/13 ; PWD=/home/nobody ; USER=root ; COMMAND=/bin/ls""", "${.app.name}", "sudo"),
    ("""<0>1 2012-03-05T15:10:34+02:00 localhost kernel 1234 - - @cim: {"name1":"value1", "name2":"value2"}""", "${.app.name}", "cim"),
    ("""<0>1 2012-03-05T15:10:34+02:00 localhost kernel 1234 - - @cee: {"name1":"value1", "name2":"value2"}""", "${.app.name}", "cee"),
]


@pytest.mark.parametrize(
    "input_message, template, expected_value", test_parameters_syslog,
    ids=list(map(str, range(len(test_parameters_syslog)))),
)
def test_application_syslog(config, syslog_ng, input_message, template, expected_value):
    config.add_include("scl.conf")

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    syslog_parser = config.create_syslog_parser(flags="syslog-protocol")
    app_parser = config.create_app_parser(topic="syslog")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template + '\n'))
    config.create_logpath(statements=[generator_source, syslog_parser, app_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == expected_value



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import json

import pytest

from src.common.network_operations import get_hostname


test_parameters = [
    (
        r"<189>29: foo: *Apr 29 13:58:40.411: %SYS-5-CONFIG_I: Configured from console by console",
        {
            "PRI": "189",
            "HOST": "foo",
            "DATE": "Apr 29 13:58:40",
            "MSEC": "411",
            "_cisco": {
                "facility": "SYS",
                "severity": "5",
                "mnemonic": "CONFIG_I",
            },
            "MSG": r"%SYS-5-CONFIG_I: Configured from console by console",
        },
    ),
    (
        r"<190>30: foo: *Apr 29 13:58:46.411: %SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 stopped - CLI initiated",
        {
            "PRI": "190",
            "HOST": "foo",
            "DATE": "Apr 29 13:58:46",
            "MSEC": "411",
            "_cisco": {
                "facility": "SYS",
                "severity": "6",
                "mnemonic": "LOGGINGHOST_STARTSTOP",
            },
            "MSG": r"%SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 stopped - CLI initiated",
        },
    ),
    (
        r"<190>31: foo: *Apr 29 13:58:46.411: %SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 started - CLI initiated",
        {
            "PRI": "190",
            "HOST": "foo",
            "DATE": "Apr 29 13:58:46",
            "MSEC": "411",
            "_cisco": {
                "facility": "SYS",
                "severity": "6",
                "mnemonic": "LOGGINGHOST_STARTSTOP",
            },
            "MSG": r"%SYS-6-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 started - CLI initiated",
        },
    ),
    (
        r"<189>35: *Apr 29 14:00:16.059: %SYS-5-CONFIG_I: Configured from console by console",
        {
            "PRI": "189",
            "HOST": get_hostname(),
            "DATE": "Apr 29 14:00:16",
            "MSEC": "059",
            "_cisco": {
                "facility": "SYS",
                "severity": "5",
                "mnemonic": "CONFIG_I",
            },
            "MSG": r"%SYS-5-CONFIG_I: Configured from console by console",
        },
    ),
    (
        r"<190>32: foo: *Apr 29 13:58:46.411: %SYSMGR-STANDBY-3-SHUTDOWN_START: The System Manager has started the shutdown procedure.",
        {
            "PRI": "190",
            "HOST": "foo",
            "DATE": "Apr 29 13:58:46",
            "MSEC": "411",
            "_cisco": {
                "facility": "SYSMGR-STANDBY",
                "severity": "3",
                "mnemonic": "SHUTDOWN_START",
            },
            "MSG": r"%SYSMGR-STANDBY-3-SHUTDOWN_START: The System Manager has started the shutdown procedure.",
        },
    ),
    (
        r"<180>782431: machine1: .Nov 18 21:03:22.631 GMT: %CDP-4-NATIVE_VLAN_MISMATCH: Native VLAN mismatch discovered on TenGigabitEthernet.",
        {
            "PRI": "180",
            "HOST": "machine1",
            "DATE": "Nov 18 21:03:22",
            "MSEC": "631",
            "_cisco": {
                "facility": "CDP",
                "severity": "4",
                "mnemonic": "NATIVE_VLAN_MISMATCH",
            },
            "MSG": r"%CDP-4-NATIVE_VLAN_MISMATCH: Native VLAN mismatch discovered on TenGigabitEthernet.",
        },
    ),
    (
        r"<166>2022-02-16T15:31:53Z na-zy-int-fp1140-p02 : %FTD-6-305012: Teardown dynamic TCP translation from FOO-WAN_IN:10.44.60.80/59877 to FOO-OUTSIDE:6.7.8.9/59877 duration 0:01:01",
        {
            "PRI": "166",
            "HOST": "na-zy-int-fp1140-p02",
            "DATE": "Feb 16 15:31:53",
            "MSEC": "000",
            "_cisco": {
                "facility": "FTD",
                "severity": "6",
                "mnemonic": "305012",
            },
            "MSG": r"%FTD-6-305012: Teardown dynamic TCP translation from FOO-WAN_IN:10.44.60.80/59877 to FOO-OUTSIDE:6.7.8.9/59877 duration 0:01:01",
        },
    ),
    (
        r"<190>123030: some-remote-host: %SYS-5-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 stopped - CLI initiated",
        {
            "PRI": "190",
            "HOST": "some-remote-host",
            "_cisco": {
                "facility": "SYS",
                "severity": "5",
                "mnemonic": "LOGGINGHOST_STARTSTOP",
            },
            "MSG": r"%SYS-5-LOGGINGHOST_STARTSTOP: Logging to host 192.168.1.239 stopped - CLI initiated",
        },
    ),
    (
        r"<187>138076: RP/0/RP0/CPU0:Dec 11 12:43:29.227 EST: snmpd[1002]: %SNMP-SNMP-3-AUTH_FAIL : Received snmp request on unknown community from 0.0.0.0",
        {
            "PRI": "187",
            "_cisco": {
                "facility": "SNMP-SNMP",
                "severity": "3",
                "mnemonic": "AUTH_FAIL",
                "cpu_module": "RP/0/RP0/CPU0",
            },
            "MSG": r"%SNMP-SNMP-3-AUTH_FAIL : Received snmp request on unknown community from 0.0.0.0",
        },
    ),
    (
        r"<187>3408: CLC 6/0: Dec 11 13:31:14.214 EST: %PKI-3-CERTIFICATE_INVALID_EXPIRED: Certificate chain validation has failed.  The certificate (SN: XXXXXXXX) has expired.    Validity period ended on 2025-01-23T00:00:00Z",
        {
            "PRI": "187",
            "_cisco": {
                "facility": "PKI",
                "severity": "3",
                "mnemonic": "CERTIFICATE_INVALID_EXPIRED",
                "cpu_module": "CLC 6/0",
            },
            "MSG": r"%PKI-3-CERTIFICATE_INVALID_EXPIRED: Certificate chain validation has failed.  The certificate (SN: XXXXXXXX) has expired.    Validity period ended on 2025-01-23T00:00:00Z",
        },
    ),
]


@pytest.mark.parametrize(
    "input_message, expected_kv_pairs",
    test_parameters,
    ids=range(len(test_parameters)),
)
def test_cisco_parser(config, syslog_ng, input_message, expected_kv_pairs):
    config.add_include("scl.conf")

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    cisco_parser = config.create_cisco_parser()
    file_destination = config.create_file_destination(
        file_name="output.log",
        template=config.stringify("$(format-json --scope all-nv-pairs PRI DATE MSEC MSG)" + "\n"),
    )
    config.create_logpath(statements=[generator_source, cisco_parser, file_destination])

    syslog_ng.start(config)

    output_json = json.loads(file_destination.read_log())

    assert output_json["PRI"] == expected_kv_pairs["PRI"]
    if "HOST" in expected_kv_pairs:
        assert output_json["HOST"] == expected_kv_pairs["HOST"]
    if "DATE" in expected_kv_pairs:
        assert output_json["DATE"] == expected_kv_pairs["DATE"]
    if "MSEC" in expected_kv_pairs:
        assert output_json["MSEC"] == expected_kv_pairs["MSEC"]
    assert output_json["_cisco"]["facility"] == expected_kv_pairs["_cisco"]["facility"]
    assert output_json["_cisco"]["severity"] == expected_kv_pairs["_cisco"]["severity"]
    assert output_json["_cisco"]["mnemonic"] == expected_kv_pairs["_cisco"]["mnemonic"]
    if "cpu_module" in expected_kv_pairs["_cisco"]:
        assert output_json["_cisco"]["cpu_module"] == expected_kv_pairs["_cisco"]["cpu_module"]
    assert output_json["MSG"] == expected_kv_pairs["MSG"]



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import annotations
from pathlib import Path

import yaml

import cdn
import remote_storage_synchronizer

from typing import Type


class Config:
    """
    A class that helps to configure the indexer.

    The config must be written in yaml format.

    There are 3 mandatory fields:
      * `storage`: configures the incoming and indexed remote storage synchronizers
      * `cdn`: configures the CDN
      * `gpg-key-path`: configures the path of the GPG key, needed for signing

    There are multiple vendor implementations for `storage` and `cdn`.
    Choosing one must be done with the use of the `vendor` field.

    `storage` is further split to `incoming` and `indexed` fields.

    The next level for both `storage` and `cdn` is regarding the suite.
    There are 3 different settings here:
      * `all`: sets the options for both stable and nightly suites
      * `stable`: sets the options for the stable suite
      * `nightly`: sets the options for the nightly suite

    The specific vendor's options are custom for each implementation.
    These options must be set under the respective suite.

    Example config:
    ```yaml
    storage:
      vendor: "azure"
      incoming:
        all:
          storage-name: "incoming"
          connection-string: "storage-secret1"
      indexed:
        stable:
          storage-name: "indexed"
          connection-string: "storage-secret2"
        nightly:
          storage-name: "indexed"
          connection-string: "storage-secret3"

    cdn:
      vendor: "azure"
      all:
        resource-group-name: "cdn-secret1"
        profile-name: "cdn-secret2"
        endpoint-name: "cdn-secret3"
        endpoint-subscription-id: "cdn-secret4"
        tenant-id: "cdn-secret5"
        client-id: "cdn-secret6"
        client-secret: "cdn-secret7"

    gpg-key-path: "secret-path"
    ```
    """

    def __init__(self, config_content: str):
        try:
            self.__cfg: dict = yaml.safe_load(config_content)
        except yaml.YAMLError:
            raise Exception("Failed to load config content.")

    @classmethod
    def from_file(cls: Type, file_path: Path) -> Config:
        with Path(file_path).open("r") as f:
            config_content = f.read()

        return cls(config_content)

    @classmethod
    def from_string(cls: Type, config_content: str) -> Config:
        return cls(config_content)

    def create_incoming_remote_storage_synchronizer(
        self,
        suite: str,
    ) -> remote_storage_synchronizer.RemoteStorageSynchronizer:
        vendor = self.__cfg["storage"]["vendor"]
        suites = self.__cfg["storage"]["incoming"]

        cls = Config.__get_storage_implementation(vendor)
        options = Config.__get_options_for_suite(suites, suite)

        return cls.from_config(options)

    def create_indexed_remote_storage_synchronizer(
        self,
        suite: str,
    ) -> remote_storage_synchronizer.RemoteStorageSynchronizer:
        vendor = self.__cfg["storage"]["vendor"]
        suites = self.__cfg["storage"]["indexed"]

        cls = Config.__get_storage_implementation(vendor)
        options = Config.__get_options_for_suite(suites, suite)

        return cls.from_config(options)

    def create_cdn(self, suite: str) -> cdn.CDN:
        vendor = self.__cfg["cdn"]["vendor"]
        suites = self.__cfg["cdn"]

        cls = Config.__get_cdn_implementation(vendor)
        options = Config.__get_options_for_suite(suites, suite)

        return cls.from_config(options)

    def get_gpg_key_path(self) -> Path:
        return self.__cfg["gpg-key-path"]

    @staticmethod
    def __get_storage_implementation(vendor: str) -> Type[remote_storage_synchronizer.RemoteStorageSynchronizer]:
        try:
            return remote_storage_synchronizer.get_implementations()[vendor]
        except KeyError:
            raise Exception("Unexpected storage vendor: {}".format(vendor))

    @staticmethod
    def __get_cdn_implementation(vendor: str) -> Type[cdn.CDN]:
        try:
            return cdn.get_implementations()[vendor]
        except KeyError:
            raise Exception("Unexpected cdn vendor: {}".format(vendor))

    @staticmethod
    def __get_options_for_suite(cfg_sub_block: dict, suite: str) -> dict:
        if suite in cfg_sub_block.keys():
            return cfg_sub_block[suite]

        return cfg_sub_block["all"]



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import argparse
import logging
import subprocess
from datetime import datetime
from pathlib import Path

import pytest

import src.testcase_parameters.testcase_parameters as tc_parameters
from src.helpers.loggen.loggen import Loggen
from src.message_builder.bsd_format import BSDFormat
from src.message_builder.log_message import LogMessage
from src.syslog_ng.syslog_ng import SyslogNg
from src.syslog_ng.syslog_ng_paths import SyslogNgPaths
from src.syslog_ng_config.syslog_ng_config import SyslogNgConfig
from src.syslog_ng_ctl.syslog_ng_ctl import SyslogNgCtl
from src.testcase_parameters.testcase_parameters import TestcaseParameters

logger = logging.getLogger(__name__)


class InstallDirAction(argparse.Action):
    def __call__(self, parser, namespace, path, option_string=None):
        install_dir = Path(path)

        if not install_dir.is_dir():
            raise argparse.ArgumentTypeError("{0} is not a valid directory".format(path))

        binary = Path(install_dir, "sbin/syslog-ng")
        if not binary.exists():
            raise argparse.ArgumentTypeError("{0} not exist".format(binary))

        setattr(namespace, self.dest, path)


# Command line options
def pytest_addoption(parser):
    parser.addoption("--runslow", action="store_true", default=False, help="run slow tests")
    parser.addoption("--run-under", help="Run syslog-ng under selected tool, example tools: [valgrind, strace]")
    parser.addoption(
        "--installdir",
        action=InstallDirAction,
        help="Set installdir for installed syslog-ng. Used when installmode is: custom. Example path: '/home/user/syslog-ng/installdir/'",
    )
    parser.addoption(
        "--reports",
        action="store",
        default=get_relative_report_dir(),
        help="Path for report files folder. Default form: 'reports/<current_date>'",
    )


def get_relative_report_dir():
    return str(Path("reports/", get_current_date()))


def get_current_date():
    return datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")


# Pytest Hooks
def pytest_collection_modifyitems(config, items):
    if config.getoption("--runslow"):
        return
    skip_slow = pytest.mark.skip(reason="need --runslow option to run")
    for item in items:
        if "slow" in item.keywords:
            item.add_marker(skip_slow)


def pytest_runtest_logreport(report):
    if report.outcome == "failed":
        logger.error("\n{}".format(report.longrepr))


# Pytest Fixtures
@pytest.fixture
def testcase_parameters(request):
    return TestcaseParameters(request)


@pytest.fixture
def config(request, teardown):
    return SyslogNgConfig(request.getfixturevalue("version"), teardown)


@pytest.fixture
def syslog_ng(request, testcase_parameters, teardown):
    tc_parameters.INSTANCE_PATH = SyslogNgPaths(testcase_parameters).set_syslog_ng_paths("server")
    syslog_ng = SyslogNg(tc_parameters.INSTANCE_PATH, testcase_parameters, teardown)
    teardown.register(syslog_ng.stop)
    return syslog_ng


class TeardownRegistry:
    teardown_callbacks = []

    def register(self, teardown_callback):
        TeardownRegistry.teardown_callbacks.append(teardown_callback)

    def execute_teardown_callbacks(self):
        for teardown_callback in TeardownRegistry.teardown_callbacks:
            teardown_callback()


@pytest.fixture(scope="function")
def teardown():
    teardown_registry = TeardownRegistry()
    yield teardown_registry
    teardown_registry.execute_teardown_callbacks()


@pytest.fixture
def syslog_ng_ctl(syslog_ng):
    return SyslogNgCtl(syslog_ng.instance_paths)


@pytest.fixture
def bsd_formatter():
    return BSDFormat()


@pytest.fixture
def log_message():
    return LogMessage()


@pytest.fixture(scope="session")
def version(request):
    installdir = request.config.getoption("--installdir")
    binary_path = str(Path(installdir, "sbin", "syslog-ng"))
    version_output = subprocess.check_output([binary_path, "--version"]).decode()
    return version_output.splitlines()[1].split()[2]


@pytest.fixture
def loggen(testcase_parameters):
    server = Loggen()
    yield server
    server.stop()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.message_builder.log_message import LogMessage


def write_msg_with_fields(file_source, bsd_formatter, hostname, program):
    log_message = LogMessage().hostname(hostname).program(program)
    input_message = bsd_formatter.format_message(log_message)
    expected_message = bsd_formatter.format_message(log_message.remove_priority())
    file_source.write_log(input_message)
    return expected_message


def test_multiple_embedded_logpaths(config, syslog_ng, bsd_formatter):
    # Check the correct output if the logpath is the following
    # log {
    #     source(s_file);
    #     log { filter{host("host-A")}; destination(d_file1); };
    #     log { filter{program("app-A")}; destination(d_file2); };
    #     log { destination(d_file3); };
    # };
    # log { destination(d_file4); };
    # input logs:
    # Oct 11 22:14:15 host-A app-A: message from host-A and app-A
    # Oct 11 22:14:15 host-A app-B: message from host-A and app-B
    # Oct 11 22:14:15 host-B app-A: message from host-B and app-A
    # Oct 11 22:14:15 host-B app-B: message from host-B and app-B

    config.update_global_options(keep_hostname="yes")

    file_source = config.create_file_source(file_name="input.log")
    host_filter = config.create_filter(host="'host-A'")
    program_filter = config.create_filter(program="'app-A'")
    file_destination1 = config.create_file_destination(file_name="output1.log")
    file_destination2 = config.create_file_destination(file_name="output2.log")
    file_destination3 = config.create_file_destination(file_name="output3.log")
    file_destination4 = config.create_file_destination(file_name="output4.log")

    second_level_logpath1 = config.create_inner_logpath(statements=[host_filter, file_destination1])

    second_level_logpath2 = config.create_inner_logpath(statements=[program_filter, file_destination2])

    second_level_logpath3 = config.create_inner_logpath(statements=[file_destination3])

    config.create_logpath(
        statements=[file_source, second_level_logpath1, second_level_logpath2, second_level_logpath3],
    )
    config.create_logpath(statements=[file_destination4])

    expected_message1 = write_msg_with_fields(file_source, bsd_formatter, "host-A", "app-A")
    expected_message2 = write_msg_with_fields(file_source, bsd_formatter, "host-A", "app-B")
    expected_message3 = write_msg_with_fields(file_source, bsd_formatter, "host-B", "app-A")
    expected_message4 = write_msg_with_fields(file_source, bsd_formatter, "host-B", "app-B")

    syslog_ng.start(config)

    dest1_logs = file_destination1.read_logs(counter=2)
    # host("host-A") matches on first and second messages
    assert expected_message1 in dest1_logs
    assert expected_message2 in dest1_logs

    dest2_logs = file_destination2.read_logs(counter=2)
    # program("app-A") matches on first and third messages
    assert expected_message1 in dest2_logs
    assert expected_message3 in dest2_logs

    dest3_logs = file_destination3.read_logs(counter=4)
    # every message should arrived into destination3
    # there is no filter() on this logpath
    assert expected_message1 in dest3_logs
    assert expected_message2 in dest3_logs
    assert expected_message3 in dest3_logs
    assert expected_message4 in dest3_logs

    # no messages should arrived into destination4,
    # no source() or flags(catch-all) is added
    assert file_destination4.get_path().exists() is False



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from enum import Enum, auto
from pathlib import Path


class WorkingDir:
    def __init__(self, root_dir: Path) -> None:
        self.__root_dir = root_dir
        self.__working_dir = root_dir

    @property
    def root_dir(self) -> Path:
        return self.__root_dir

    @property
    def working_dir(self) -> Path:
        return self.__working_dir

    def set_sub_dir(self, sub_dir: Path) -> None:
        self.__working_dir = Path(self.__root_dir, sub_dir)


class FileSyncState(Enum):
    IN_SYNC = auto()
    DIFFERENT = auto()
    NOT_IN_REMOTE = auto()
    NOT_IN_LOCAL = auto()


class RemoteStorageSynchronizer(ABC):
    def __init__(self, remote_root_dir: Path, local_root_dir: Path) -> None:
        self.remote_dir = WorkingDir(remote_root_dir)
        self.local_dir = WorkingDir(local_root_dir)

        self.__logger = RemoteStorageSynchronizer.__create_logger()

    @staticmethod
    @abstractmethod
    def get_config_keyword() -> str:
        pass

    @staticmethod
    @abstractmethod
    def from_config(cfg: dict) -> RemoteStorageSynchronizer:
        pass

    @abstractmethod
    def sync_from_remote(self) -> None:
        pass

    @abstractmethod
    def sync_to_remote(self) -> None:
        pass

    @abstractmethod
    def create_snapshot_of_remote(self) -> None:
        pass

    def set_sub_dir(self, sub_dir: Path) -> None:
        self.remote_dir.set_sub_dir(sub_dir)
        self.local_dir.set_sub_dir(sub_dir)

    @staticmethod
    def __create_logger() -> logging.Logger:
        logger = logging.getLogger("RemoteStorageSynchronizer")
        logger.setLevel(logging.DEBUG)
        return logger

    def _prepare_log(self, message: str, **kwargs: str) -> str:
        if len(kwargs) > 0:
            message += "\t{}".format(kwargs)
        return message

    def _log_info(self, message: str, **kwargs: str) -> None:
        log = self._prepare_log(message, **kwargs)
        self.__logger.info(log)

    def _log_debug(self, message: str, **kwargs: str) -> None:
        log = self._prepare_log(message, **kwargs)
        self.__logger.debug(log)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from src.common.blocking import wait_until_true
from src.common.file import copy_shared_file
from src.common.file import File
from src.common.random_id import get_unique_id

TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
INPUT_MESSAGES = "message 0\n\n"
EXPECTED_MESSAGES = "1.1.1.1 2.2.2.2 3333 4444 4 message 0\n"
NUMBER_OF_MESSAGES = 1


def test_pp_tls_passthrough(config, syslog_ng, port_allocator, loggen, testcase_parameters):
    server_key_path = copy_shared_file(testcase_parameters, "server.key")
    server_cert_path = copy_shared_file(testcase_parameters, "server.crt")

    network_source = config.create_network_source(
        ip="localhost",
        port=port_allocator(),
        transport='"proxied-tls-passthrough"',
        flags="no-parse",
        tls={
            "key-file": server_key_path,
            "cert-file": server_cert_path,
            "peer-verify": '"optional-untrusted"',
        },
    )

    file_destination = config.create_file_destination(file_name="output.log", template=TEMPLATE)
    config.create_logpath(statements=[network_source, file_destination])

    syslog_ng.start(config)

    loggen_input_file_path = Path("loggen_input_{}.txt".format(get_unique_id()))
    loggen_input_file = File(loggen_input_file_path)
    loggen_input_file.write_content_and_close(INPUT_MESSAGES)

    loggen.start(
        network_source.options["ip"], network_source.options["port"], number=NUMBER_OF_MESSAGES,
        use_ssl=True, proxied_tls_passthrough=True, read_file=str(loggen_input_file_path),
        dont_parse=True, proxy_src_ip="1.1.1.1", proxy_dst_ip="2.2.2.2", proxy_src_port="3333",
        proxy_dst_port="4444",
    )

    wait_until_true(lambda: loggen.get_sent_message_count() == NUMBER_OF_MESSAGES)
    assert file_destination.read_log() == EXPECTED_MESSAGES



#############################################################################
# Copyright (c) 2007-2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *

config = """@version: %(syslog_ng_version)s

options { ts_format(iso); chain_hostnames(no); keep_hostname(yes); threaded(yes); };

source s_int { internal(); };
source s_tcp { tcp(port(%(port_number)d)); };

destination d_messages { file("test-performance.log"); };

log { source(s_tcp); destination(d_messages); };

""" % locals()

def test_performance():
    expected_rate = {
      'bzorp': 10000
    }
    print_user("Starting loggen for 10 seconds")
    out = os.popen("../loggen/loggen --quiet --stream --inet --rate 1000000 --size 160 --interval 10 --active-connections 1 127.0.0.1 %d 2>&1 |tail -n +1" % port_number, 'r').read()

    print_user("performance: %s" % out)
    rate = float(re.sub('^.*rate = ([0-9.]+).*$', '\\1', out))

    hostname = os.uname()[1]
    if hostname in expected_rate:
        return rate > expected_rate[hostname]

    # we expect to be able to process at least 1000 msgs/sec even on our venerable HP-UX
    return rate > 100



#!/bin/env python3
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import logging
from argparse import ArgumentParser
from pathlib import Path
from sys import stdin
from typing import List

from indexer import Indexer, NightlyDebIndexer, StableDebIndexer
from config import Config


def add_required_arguments(parser: ArgumentParser) -> None:
    required_argument_group = parser.add_argument_group("required arguments")
    mutually_exclusive_group = required_argument_group.add_mutually_exclusive_group(required=True)

    mutually_exclusive_group.add_argument(
        "--config-file-path",
        type=str,
        help="The path of the config file in yaml format. Cannot be used with `--config-content`.",
    )
    mutually_exclusive_group.add_argument(
        "--config-content",
        type=str,
        help="The raw content of the config in yaml format. Cannot be used with `--config-file-path`.",
    )

    required_argument_group.add_argument(
        "--run-id",
        type=str,
        required=True,
        help='The "run-id" of the "draft-release" or "nightly-release" GitHub Actions job.',
    )
    required_argument_group.add_argument(
        "--suite",
        type=str,
        required=True,
        metavar="SUITE",
        choices=["stable", "nightly"],
        help='"stable" or "nightly".',
    )


def add_optional_arguments(parser: ArgumentParser) -> None:
    parser.add_argument(
        "--gpg-key-passphrase-from-stdin",
        action="store_true",
        help="If this option is set, the passphrase of the GPG-key file will be read from STDIN.",
    )
    parser.add_argument(
        "--log-file",
        type=str,
        help="Also log more verbosely into this file.",
    )
    parser.add_argument(
        "--flush-cache-only",
        action="store_true",
        help="Only flush the caches of the Azure CDN. Used for testing that part.",
    )


def parse_args() -> dict:
    parser = ArgumentParser()
    add_required_arguments(parser)
    add_optional_arguments(parser)

    return vars(parser.parse_args())


def init_logging(args: dict) -> None:
    handlers: List[logging.Handler] = []

    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.INFO)
    handlers.append(stream_handler)

    if "log_file" in args.keys() and args["log_file"] is not None:
        file_handler = logging.FileHandler(args["log_file"])
        file_handler.setLevel(logging.DEBUG)
        handlers.append(file_handler)

    logging.basicConfig(
        format="%(asctime)s\t[%(name)s]\t%(message)s",
        handlers=handlers,
    )


def load_config(args: dict) -> Config:
    if "config_file_path" in args.keys() and args["config_file_path"] is not None:
        return Config.from_file(Path(args["config_file_path"]))
    elif "config_content" in args.keys() and args["config_content"] is not None:
        return Config.from_string(args["config_content"])
    else:
        raise KeyError("Missing config related option.")


def construct_indexers(cfg: Config, args: dict) -> List[Indexer]:
    suite = args["suite"]

    incoming_remote_storage_synchronizer = cfg.create_incoming_remote_storage_synchronizer(suite)
    indexed_remote_storage_synchronizer = cfg.create_indexed_remote_storage_synchronizer(suite)
    cdn = cfg.create_cdn(suite)

    gpg_key_path = Path(cfg.get_gpg_key_path())
    gpg_key_passphrase = stdin.read() if args["gpg_key_passphrase_from_stdin"] else None

    indexers: List[Indexer] = []

    if suite == "nightly":
        indexers.append(
            NightlyDebIndexer(
                incoming_remote_storage_synchronizer=incoming_remote_storage_synchronizer,
                indexed_remote_storage_synchronizer=indexed_remote_storage_synchronizer,
                cdn=cdn,
                run_id=args["run_id"],
                gpg_key_path=gpg_key_path,
                gpg_key_passphrase=gpg_key_passphrase,
            )
        )
    elif suite == "stable":
        indexers.append(
            StableDebIndexer(
                incoming_remote_storage_synchronizer=incoming_remote_storage_synchronizer,
                indexed_remote_storage_synchronizer=indexed_remote_storage_synchronizer,
                cdn=cdn,
                run_id=args["run_id"],
                gpg_key_path=gpg_key_path,
                gpg_key_passphrase=gpg_key_passphrase,
            )
        )
    else:
        raise NotImplementedError("Unexpected suite: {}".format(suite))

    return indexers


def main() -> None:
    args = parse_args()
    init_logging(args)

    cfg = load_config(args)

    indexers = construct_indexers(cfg, args)
    if args["flush_cache_only"]:
        for indexer in indexers:
            indexer.flush_cdn_cache()
    else:
        for indexer in indexers:
            indexer.index()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

from src.helpers.snmptrapd.conftest import *  # noqa:F403, F401


@pytest.mark.snmp
def test_snmp_dest_missing_snmp_obj(config, syslog_ng, snmptrapd, snmp_test_params):
    generator_source = config.create_example_msg_generator_source(num=1)
    snmp_destination = config.create_snmp_destination(
        host=snmp_test_params.get_ip_address(),
        port=snmptrapd.get_port(),
        trap_obj=snmp_test_params.get_basic_trap_obj(),
    )
    config.create_logpath(statements=[generator_source, snmp_destination])

    syslog_ng.start(config)

    expected_traps = snmp_test_params.get_expected_empty_trap()
    assert expected_traps == snmptrapd.get_traps(len(expected_traps))



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from dataclasses import dataclass

from src.syslog_ng_config.statements.logpath.logpath import LogPath


@dataclass
class ExpectedLogPathStats:
    logpath: LogPath
    ingress: int
    egress: int


def assert_logpath_stats(logpath, ingress, egress):
    stats = logpath.get_prometheus_stats()

    found = False
    for stat in stats:
        if stat.name == "syslogng_route_ingress_total" and stat.labels == {"id": logpath.name}:
            found = True
            actual_ingress = stat.value
    assert found, "Did not find ingress stats counter for logpath: %s" % (logpath.name)

    found = False
    for stat in stats:
        if stat.name == "syslogng_route_egress_total" and stat.labels == {"id": logpath.name}:
            found = True
            actual_egress = stat.value
    assert found, "Did not find egress stats counter for logpath: %s" % (logpath.name)

    assert actual_ingress == ingress, "Unexpected ingress stat. Logpath: %s Expected: %d Actual: %d" % (logpath.name, ingress, actual_ingress)
    assert actual_egress == egress, "Unexpected egress stat. Logpath: %s Expected: %d Actual: %d" % (logpath.name, egress, actual_egress)


def assert_multiple_logpath_stats(expected_logpath_stats):
    for _, expected_logpath_stat in expected_logpath_stats.items():
        assert_logpath_stats(expected_logpath_stat.logpath, expected_logpath_stat.ingress, expected_logpath_stat.egress)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2024 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_config.renderer import render_statement


def test_multiple_filters_implicit_and(config, syslog_ng):
    file_true = config.create_file_destination(file_name="dest-true.log", template="\"$MSG\\n\"")
    file_false = config.create_file_destination(file_name="dest-false.log", template="\"$MSG\\n\"")

    preamble = f"""
@version: {config.get_version()}

options {{ stats(level(1)); }};

source genmsg {{
    example-msg-generator(num(1) template("MESSAGE"));
    example-msg-generator(num(1) template("foobar"));
}};

filter f_filter {{
    not program("xyz");
    message("MESSAGE");
}};

destination dest_true {{
{render_statement(file_true)};
}};

destination dest_false {{
{render_statement(file_false)};
}};

log {{
    source(genmsg);
    if {{
        filter(f_filter);
        destination(dest_true);
    }} else {{
        destination(dest_false);
    }};
}};
"""
    config.set_raw_config(preamble)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert file_false.get_stats()["processed"] == 1

    assert "MESSAGE" in file_true.read_log()
    assert "foobar" in file_false.read_log()



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from .test_lexer import TestLexer
from ..getoptlexer import GetoptLexer
from ..commandlinelexer import CommandLineLexer


class TestGetoptLexer(TestLexer):
    def setUp(self):
        self._known_commands = ['print', 'foo-bar']
        self._aliases = {
            'p': 'print',
            'fb': 'foo-bar'
        }
        self._lexer = self._construct_lexer()

    # pylint: disable=arguments-differ
    def _construct_lexer(self, known_commands=None, known_options=None, aliases=None):
        return GetoptLexer(CommandLineLexer(),
                           known_commands=known_commands or self._known_commands,
                           known_options=known_options,
                           aliases=aliases or self._aliases)

    def test_lexer_returns_command_token_for_an_unknown_command_as_the_first_token(self):
        self._lexer.input("unknown-cmd")
        token = self._next_token()
        self.assertEqual(token.type, "COMMAND")

    def test_lexer_returns_specific_token_for_known_commands(self):
        for command in self._known_commands:
            self._lexer.input(command)
            self._assert_next_token_type_equals(f"COMMAND_{command.upper().replace('-', '_')}")

    def test_lexer_translates_aliases(self):
        for (alias, command) in self._aliases.items():
            self._lexer.input(alias)
            self._assert_next_token_type_equals(f"COMMAND_{command.upper().replace('-', '_')}")

    def test_known_commands_are_not_returned_as_tokens_for_non_first_arguments(self):
        self._lexer.input("print print")
        self.assertEqual(self._next_token().type, "COMMAND_PRINT")
        self.assertEqual(self._next_token().type, "ARG")

    def test_double_quoted_arguments_are_returned_as_a_single_token(self):
        self._lexer.input('print "foo bar"')
        self.assertEqual(self._next_token().type, "COMMAND_PRINT")
        token = self._next_token()
        self.assertEqual(token.type, "ARG")
        self.assertEqual(token.value, "foo bar")

    def test_lexer_returns_token_in_a_sequence(self):
        self._lexer.input('''print foo bar''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals('foo')
        self._assert_next_arg_equals('bar')
        self._assert_last_token_reached()

    def test_lexer_handles_quoted_args_by_unqouting_them(self):
        self._lexer.input('''print "foo bar" 'bar foo' ''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals('foo bar')
        self._assert_next_arg_equals('bar foo')
        self._assert_last_token_reached()

    def test_lexer_handles_parenthesized_expression_as_a_single_token(self):
        self._lexer.input('''print (''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals("(")

        self._lexer.input('''print ( )''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals("( )")

        self._lexer.input('''print (1 + 1)''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals("(1 + 1)")

        self._lexer.input('''print ) )''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals(")")
        self._assert_next_arg_equals(")")

        self._lexer.input('''print ($MSG == foobar)''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_arg_equals("($MSG == foobar)")

    def test_lexer_translates_args_to_option_tokens_if_they_are_known(self):
        self._lexer = self._construct_lexer(known_options=("--foo", "--bar", "-a", "-b", "abc"))
        self._lexer.input('''print --foo --bar -a -b abc''')
        self._assert_next_token_type_equals("COMMAND_PRINT")
        self._assert_next_token_type_equals("OPT__FOO")
        self._assert_next_token_type_equals("OPT__BAR")
        self._assert_next_token_type_equals("OPT_A")
        self._assert_next_token_type_equals("OPT_B")
        self._assert_next_token_type_equals("OPTABC")

    def test_lexer_doesnt_translate_command_token_as_an_option(self):
        self._lexer = self._construct_lexer(known_options=("--foo", "--bar", "-a", "-b"))
        self._lexer.input('''--foo --bar -a -b''')
        self._assert_next_token_equals("COMMAND", value="--foo")

    def _assert_last_token_reached(self):
        token = self._next_token()
        self.assertIsNone(token)

    def _assert_next_arg_equals(self, token_value):
        token = self._next_token()
        self.assertEqual(token.type, "ARG")
        self.assertEqual(token.value, token_value)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest


test_parameters = [
    ("$(format-json test.*)\n", """{"test":{"key2":"value2","key1":"value1"}}\n"""),
    # transformations
    ("$(format-json test.* --add-prefix foo.)\n", """{"foo":{"test":{"key2":"value2","key1":"value1"}}}\n"""),
    ("$(format-json test.* --replace-prefix test=foobar)\n", """{"foobar":{"key2":"value2","key1":"value1"}}\n"""),
    ("$(format-json test.* --shift-levels 1)\n", """{"key2":"value2","key1":"value1"}\n"""),
    ("$(format-json test.* --shift 2)\n", """{"st":{"key2":"value2","key1":"value1"}}\n"""),
    ("$(format-json test.* --upper)\n", """{"TEST":{"KEY2":"value2","KEY1":"value1"}}\n"""),
    ("$(format-json MESSAGE --lower)\n", """{"message":"-- Generated message. --"}\n"""),
]


@pytest.mark.parametrize(
    "template, expected_value", test_parameters,
    ids=list(map(lambda x: x[0], test_parameters)),
)
def test_value_pairs(config, syslog_ng, template, expected_value):

    generator_source = config.create_example_msg_generator_source(num=1, values="test.key1 => value1 test.key2 => value2")
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify(template))

    config.create_logpath(statements=[generator_source, file_destination])
    syslog_ng.start(config)
    log = file_destination.read_log()
    assert log == expected_value
